from bs4 import BeautifulSoup, Tag
import pandas as pd
from bs4.element import ResultSet
import requests as req


def table_rows(soup: BeautifulSoup, table_id: str) -> ResultSet:
    table = soup.find(id=table_id)
    if table:
        table_body = table.find("tbody")
        rows = table_body.find_all("tr")
        rows = filter(lambda r: r.find("td"), rows)  # Filter headers in middle
        return rows
    return []


def read_table_to_df(soup: BeautifulSoup, table_id: str) -> pd.DataFrame:
    table = soup.find(id=table_id)
    if table:
        # Getting first because id should be unique
        table_df = pd.read_html(str(table))[0]
        return table_df
    return pd.DataFrame()


def get_soup(url: str) -> BeautifulSoup:
    r = req.get(url)
    if r.status_code == 200:
        return BeautifulSoup(r.text, "html.parser")
    else:
        raise Exception("Problem with HTTP request.")


def has_attr_with_val(tag: Tag, attr: str, val: str) -> bool:
    return tag.has_attr(attr) and tag[attr] == val


def find_data_stat(html_tag: Tag, val: str) -> Tag:
    return html_tag.find(lambda x: has_attr_with_val(x, "data-stat", val))


def parse_data_stat(html_tag: Tag, val: str, type: type):
    stat = find_data_stat(html_tag, val)
    if stat:
        stat = stat.text
    if stat:
        return type(stat)
    return None



from bs4.element import Tag
from puck_me.lib import utils
from puck_me.game.game import Game


class GoalieGame(Game):
    def __init__(self, row_html: Tag, is_playoff: bool) -> None:
        super().__init__(row_html, is_playoff)
        self.__populate_goalie_stats(row_html)

    def __str__(self) -> str:
        pub_dict = dict()
        for (key, value) in self.__dict__.items():
            if not key.startswith("_GoalieGame"):
                pub_dict[key] = value
        return str(pub_dict)

    def __populate_goalie_stats(self, row_html: Tag):
        self.decision = utils.parse_data_stat(row_html, "decision", str)
        self.decision = utils.parse_data_stat(row_html, "goals_against", int)
        self.decision = utils.parse_data_stat(row_html, "shots_against", int)
        self.decision = utils.parse_data_stat(row_html, "saves", int)
        self.decision = utils.parse_data_stat(row_html, "save_pct", float)
        self.decision = utils.parse_data_stat(row_html, "shutouts", int)



from puck_me.goalie.goalies import Goalies
from puck_me.goalie.goalie import Goalie
from puck_me.skater.skaters import Skaters
from puck_me.skater.skater import Skater


class Players:
    @staticmethod
    def all_goalies(year: str = None) -> list[Goalie]:
        return Goalies.all_goalies(year)

    @staticmethod
    def all_skaters(year: str = None) -> list[Skater]:
        return Skaters.all_skaters(year)



from bs4.element import Tag
from puck_me.lib import utils


class Game:
    # arena
    HOME = "Home"
    AWAY = "Away"
    # result
    WIN = "Win"
    LOSS = "Loss"
    LOSS_OT = "Loss OT"
    LOSS_SO = "Loss SO"

    def __init__(self, row_html: Tag, is_playoff: bool) -> None:
        self.__base_populate(row_html)
        self.is_playoff = is_playoff

    def __base_populate(self, row_html: Tag) -> None:
        date_game = utils.find_data_stat(row_html, "date_game")
        self.date = date_game.text
        self.__url = date_game.find("a")["href"]
        self.age = utils.find_data_stat(row_html, "age").text
        team_id = utils.find_data_stat(row_html, "team_id")
        self.team = team_id.text
        self.__team_url = team_id.find("a")["href"]
        game_location = utils.find_data_stat(row_html, "game_location").text
        self.arena = Game.AWAY if game_location == "@" else Game.HOME
        opp_id = utils.find_data_stat(row_html, "opp_id")
        self.opponent = opp_id.text
        self.__opponent_url = opp_id.find("a")["href"]
        game_result = utils.find_data_stat(row_html, "game_result").text
        self.result = Game.__map_result(game_result)
        self.is_win = self.result == Game.WIN
        self.pim = int(utils.find_data_stat(row_html, "pen_min").text)
        self.time_on_ice = utils.find_data_stat(row_html, "time_on_ice").text

    @staticmethod
    def __map_result(game_result: str) -> str:
        if game_result == "W":
            return Game.WIN
        elif game_result == "L":
            return Game.LOSS
        elif game_result == "L-OT":
            return Game.LOSS_OT
        elif game_result == "L-SO":
            return Game.LOSS_SO



from enum import Enum


class SplitType(Enum):
    SEASON = "Season"
    ARENA = "Place"
    ALL_STAR = "All_Star"
    RESULT = "Result"
    MONTH = "Month"
    CONFERENCE = "Conference"
    DIVISION = "Division"
    OPPONENT = "Opponent"



from bs4.element import Tag
from puck_me.lib import utils
from puck_me.game.game import Game


class SkaterGame(Game):
    def __init__(self, row_html: Tag, is_playoff: bool) -> None:
        super().__init__(row_html, is_playoff)
        self.__populate_skater_stats(row_html)

    def __str__(self) -> str:
        pub_dict = dict()
        for (key, value) in self.__dict__.items():
            if not key.startswith("_SkaterGame"):
                pub_dict[key] = value
        return str(pub_dict)

    def __populate_skater_stats(self, row_html: Tag) -> None:
        self.goals = utils.parse_data_stat(row_html, "goals", int)
        self.goals_ev = utils.parse_data_stat(row_html, "goals_ev", int)
        self.goals_pp = utils.parse_data_stat(row_html, "goals_pp", int)
        self.goals_sh = utils.parse_data_stat(row_html, "goals_sh", int)
        self.goals_gw = utils.parse_data_stat(row_html, "goals_gw", int)
        self.assists = utils.parse_data_stat(row_html, "assists", int)
        self.assists_ev = utils.parse_data_stat(row_html, "assists_ev", int)
        self.assists_pp = utils.parse_data_stat(row_html, "assists_pp", int)
        self.assists_sh = utils.parse_data_stat(row_html, "assists_sh", int)
        self.points = utils.parse_data_stat(row_html, "points", int)
        self.plus_minus = utils.parse_data_stat(row_html, "plus_minus", int)
        self.shots = utils.parse_data_stat(row_html, "shots", int)
        self.shot_pct = utils.parse_data_stat(row_html, "shot_pct", float)
        self.shifts = utils.parse_data_stat(row_html, "shifts", int)
        self.hits = utils.parse_data_stat(row_html, "hits_all", int)
        self.blocks = utils.parse_data_stat(row_html, "blocks_all", int)
        self.faceoff_wins = utils.parse_data_stat(row_html, "faceoff_wins_all", int)
        self.faceoff_loss = utils.parse_data_stat(row_html, "faceoff_losses_all", int)
        self.faceoff_pct = utils.parse_data_stat(
            row_html, "faceoff_percentage_all", float
        )



from puck_me.player.player import Player
from bs4.element import ResultSet
from puck_me.goalie.goalie_game import GoalieGame
import pandas as pd
from enum import Enum
from puck_me.lib.split_types import SplitType


class SplitCol(Enum):
    VALUE = "Value"
    GAMES_PLAYED = "GP"
    WINS = "W"
    LOSSES = "L"
    TIE_LOSSES = "T/O"
    GOALS_AGAINST = "GA"
    SHOTS_FACED = "SA"
    SAVES = "SV"
    SAVE_PERCENT = "SV%"
    GOALS_AGAINST_AVG = "GAA"
    SHUTOUTS = "SO"
    PENALTY_MINS = "PIM"
    TIME_ON_ICE = "TOI"
    GOALS_AGAINST_EV = "EV GA"
    GOALS_AGAINST_PP = "PP GA"
    GOALS_AGAINST_SH = "SH GA"


class GoalieSplit:
    def __init__(self, split_table_series: pd.Series) -> None:
        self.value = split_table_series[SplitCol.VALUE.value]
        self.games_played = split_table_series[SplitCol.GAMES_PLAYED.value]
        self.wins = split_table_series[SplitCol.WINS.value]
        self.losses = split_table_series[SplitCol.LOSSES.value]
        self.tie_losses = split_table_series[SplitCol.TIE_LOSSES.value]
        self.goals_against = split_table_series[SplitCol.GOALS_AGAINST.value]
        self.shots_faced = split_table_series[SplitCol.SHOTS_FACED.value]
        self.saves = split_table_series[SplitCol.SAVES.value]
        self.save_percent = split_table_series[SplitCol.SAVE_PERCENT.value]
        self.goals_against_avg = split_table_series[SplitCol.GOALS_AGAINST_AVG.value]
        self.shutouts = split_table_series[SplitCol.SHUTOUTS.value]
        self.penalty_mins = split_table_series[SplitCol.PENALTY_MINS.value]
        self.time_on_ice = split_table_series[SplitCol.TIME_ON_ICE.value]
        self.goals_against_ev = split_table_series[SplitCol.GOALS_AGAINST_EV.value]
        self.goals_against_pp = split_table_series[SplitCol.GOALS_AGAINST_PP.value]
        self.goals_against_sh = split_table_series[SplitCol.GOALS_AGAINST_SH.value]


class Goalie(Player):

    ##########################
    # Public Functions
    ##########################

    def gamelog(self, year: str = None) -> list[GoalieGame]:
        if not year:
            year = self._year
        if not self._gamelog_regular_season and not self._gamelog_playoffs:
            self.__populate_gamelogs(year)
        return self._gamelog_regular_season + self._gamelog_playoffs

    def gamelog_playoffs(self, year: str = None) -> list[GoalieGame]:
        if not year:
            year = self._year
        if not self._gamelog_regular_season and not self._gamelog_playoffs:
            self.__populate_gamelogs(year)
        return self._gamelog_playoffs

    def gamelog_regular_season(self, year: str = None) -> list[GoalieGame]:
        if not year:
            year = self._year
        if not self._gamelog_regular_season and not self._gamelog_playoffs:
            self.__populate_gamelogs(year)
        return self._gamelog_regular_season

    def games_played(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.games_played

    def wins(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.wins

    def losses(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.losses

    def tie_losses(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.tie_losses

    def goals_against(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.goals_against

    def shots_faced(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.shots_faced

    def saves(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.saves

    def save_percent(self, year: str = None) -> float:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.save_percent

    def goals_against_avg(self, year: str = None) -> float:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.goals_against_avg

    def shutouts(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.shutouts

    def penalty_mins(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.penalty_mins

    def time_on_ice(self, year: str = None) -> str:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.time_on_ice

    def goals_against_ev(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.goals_against_ev

    def goals_against_pp(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.goals_against_pp

    def goals_against_sh(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.goals_against_sh

    def splits(self, split_type: SplitType, year: str = None) -> list[GoalieSplit]:
        split_df = self._split_df_lookup(year, split_type)
        return self.__split_df_to_rows(split_df)

    ##########################
    # Gamelog Section
    ##########################

    def __populate_gamelogs(self, year: str) -> None:
        if not self._gamelog_playoffs or not self._gamelog_regular_season:
            soup = self._request_player_gamelog_page(year)
            reg_season_rows = self._regular_season_games(soup)
            playoff_rows = self._playoff_games(soup)
            self._gamelog_regular_season = self.__get_games_by_table_rows(
                reg_season_rows, is_playoff=False
            )
            self._gamelog_playoffs = self.__get_games_by_table_rows(
                playoff_rows, is_playoff=True
            )

    def __get_games_by_table_rows(
        self, rows: ResultSet, is_playoff: bool
    ) -> list[GoalieGame]:
        games = []
        for row_html in rows:
            game = GoalieGame(row_html, is_playoff)
            games += [game]
        return games

    ##########################
    # Splits Section
    ##########################

    def __split_df_to_rows(self, split_df: pd.DataFrame) -> list[GoalieSplit]:
        rows = []
        for index, series in split_df.iterrows():
            rows += [GoalieSplit(series)]
        return rows



from bs4.element import Comment, ResultSet
from puck_me.hock_ref import HockRef
from bs4 import BeautifulSoup
from puck_me.lib import utils
from puck_me.skater.skater_game import SkaterGame
import pandas as pd
from enum import Enum
from puck_me.lib.split_types import SplitType


class Player:
    def __init__(self, name: str, url: str, year: str) -> None:
        self.__plr_base_url = url[1:].removesuffix(".html")
        self._gamelog_regular_season = []
        self._gamelog_playoffs = []
        self._split_dfs_dict = dict()
        self._season_split = None
        self._year = year
        self.name = name

    ##########################
    # Gamelog Section
    ##########################

    def _request_player_gamelog_page(self, year: str) -> BeautifulSoup:
        url = f"{HockRef.base_url}{self.__plr_base_url}/gamelog/{year}"
        return utils.get_soup(url)

    def _regular_season_games(self, soup: BeautifulSoup) -> ResultSet:
        table_id = "gamelog"
        return utils.table_rows(soup, table_id)

    def _playoff_games(self, soup: BeautifulSoup) -> ResultSet:
        table_id = "gamelog_playoffs"
        table = soup.find(id="all_gamelog_playoffs")
        if table:  # False if no playoff games
            # Playoff table is in comments. Need to parse the comments as soup
            for element in table(text=lambda text: isinstance(text, Comment)):
                soup = BeautifulSoup(element, "html.parser")
            return utils.table_rows(soup, table_id)
        return []

    ##########################
    # Splits Section
    ##########################

    def __request_player_splits_page(self, year: str) -> BeautifulSoup:
        url = f"{HockRef.base_url}{self.__plr_base_url}/splits/{year}"
        return utils.get_soup(url)

    def __get_splits_df(self, year: str) -> pd.DataFrame:
        soup = self.__request_player_splits_page(year)
        splits_df = utils.read_table_to_df(soup, table_id="splits")
        if not splits_df.empty:
            # Filter header rows in middle of table
            first_col = splits_df.columns.values[0]
            splits_df = splits_df[splits_df[first_col] != first_col]
        return splits_df

    def __populate_split_dfs(self, year: str) -> None:
        if not year:
            raise Exception("Year must be specified.")
        split_col = "Split"
        df = self.__get_splits_df(year)
        if df.empty:
            raise ValueError(
                f"No table found for {self.name}'s {year} splits. Check if year is valid."
            )
        curr_table = "Season"
        curr_rows = []
        df_dict = dict()
        for index, row in df.iterrows():
            if str(row.loc[split_col]) != "nan":
                df_dict[curr_table] = pd.DataFrame(curr_rows)
                curr_table = row.loc[split_col]
                curr_rows = [row]
            else:
                curr_rows += [row]
        df_dict[curr_table] = pd.DataFrame(curr_rows)
        curr_table = row.loc[split_col]
        curr_rows = [row]
        self._split_dfs_dict[year] = df_dict

    def _split_df_lookup(
        self,
        year: str,
        split_type: SplitType,
    ) -> pd.DataFrame:
        if not year:
            year = self._year
        if not self._split_dfs_dict or year not in self._split_dfs_dict.keys():
            self.__populate_split_dfs(year)
        return self._split_dfs_dict[year][split_type.value]

    ##########################
    # Main Section
    ##########################

    def _request_player_main_page(self) -> BeautifulSoup:
        url = f"{HockRef.base_url}{self.__plr_base_url}.html"
        return utils.get_soup(url)



from bs4.element import Tag
from bs4 import BeautifulSoup
from puck_me.goalie.goalie import Goalie
from puck_me.hock_ref import HockRef
from puck_me.lib import utils


class Goalies:
    @staticmethod
    def all_goalies(year: str = None) -> list[Goalie]:
        goalies = []
        if not year:
            for acsii_as_int in range(97, 123):
                letter = chr(acsii_as_int)
                if letter == "x":
                    continue  # No players w last name X
                soup = Goalies.__request_letter_player_page(letter)
                goalies += Goalies.__parse_letter_goalie_soup(soup)
            return goalies
        else:
            soup = Goalies.__request_year_goalie_page(year)
            ids_found = set()
            for row in utils.table_rows(soup, "stats"):
                goalie = Goalies.__parse_year_goalie_row(row, ids_found, year)
                if goalie:
                    goalies.append(goalie)
            return goalies

    #########################
    # Goalies by year
    #########################

    @staticmethod
    def __request_year_goalie_page(year) -> BeautifulSoup:
        url = f"{HockRef.base_url}leagues/NHL_{year}_goalies.html"
        return utils.get_soup(url)

    @staticmethod
    def __parse_year_goalie_row(plr_row: Tag, ids_found: set, year: str) -> Goalie:
        plr_id = plr_row.find(
            lambda x: utils.has_attr_with_val(x, "data-stat", "ranker")
        ).text
        if plr_id in ids_found:
            return None
        ids_found.add(plr_id)
        name_data = plr_row.find(
            lambda x: utils.has_attr_with_val(x, "data-stat", "player")
        )
        name = name_data.text.removesuffix("*")
        url = name_data.find("a")["href"]
        goalie = Goalie(name, url, year)
        return goalie

    ##############################
    # Players by letter of name
    ##############################

    @staticmethod
    def __request_letter_player_page(letter) -> BeautifulSoup:
        url = f"{HockRef.base_url}players/{letter}/"
        return utils.get_soup(url)

    @staticmethod
    def __parse_letter_goalie_soup(soup: BeautifulSoup) -> list[Goalie]:
        goalies = []
        lines = soup.find(id="div_players").find_all("p")
        for line in lines:
            class_attr = line["class"][0]
            if class_attr != "nhl" or "G" not in line.text:
                continue  # Ignore Skaters and non-nhl players
            a_tag = line.find("a")
            name = a_tag.text
            url = a_tag["href"]
            goalies += [Goalie(name, url, year=None)]
        return goalies



from puck_me.player.player import Player
from bs4.element import ResultSet
from puck_me.skater.skater_game import SkaterGame
import pandas as pd
from enum import Enum
from puck_me.lib.split_types import SplitType


class SplitCol(Enum):
    VALUE = "Value"
    GAMES_PLAYED = "GP"
    GOALS = "G"
    ASSISTS = "A"
    POINTS = "PTS"
    PLUS_MINUS = "+/-"
    PENALTY_MINS = "PIM"
    GOALS_EV = "EV"
    GOALS_PP = "PP"
    GOALS_SH = "SH"
    GOALS_GW = "GW"
    SHOTS = "S"
    SHOOTING_PCT = "S%"
    SHIFTS = "SHFT"
    TIME_ON_ICE = "TOI"
    TIME_ON_ICE_AVG = "ATOI"


class SkaterSplit:
    def __init__(self, split_table_series: pd.Series) -> None:
        self.value = split_table_series[SplitCol.VALUE.value]
        self.games_played = split_table_series[SplitCol.GAMES_PLAYED.value]
        self.goals = split_table_series[SplitCol.GOALS.value]
        self.assists = split_table_series[SplitCol.ASSISTS.value]
        self.points = split_table_series[SplitCol.POINTS.value]
        self.plus_minus = split_table_series[SplitCol.PLUS_MINUS.value]
        self.penalty_minutes = split_table_series[SplitCol.PENALTY_MINS.value]
        self.goals_ev = split_table_series[SplitCol.GOALS_EV.value]
        self.goals_pp = split_table_series[SplitCol.GOALS_PP.value]
        self.goals_sh = split_table_series[SplitCol.GOALS_SH.value]
        self.goals_gw = split_table_series[SplitCol.GOALS_GW.value]
        self.shots = split_table_series[SplitCol.SHOTS.value]
        self.shooting_pct = split_table_series[SplitCol.SHOOTING_PCT.value]
        self.shifts = split_table_series[SplitCol.SHIFTS.value]
        self.time_on_ice_total = split_table_series[SplitCol.TIME_ON_ICE.value]
        self.time_on_ice_average = split_table_series[SplitCol.TIME_ON_ICE_AVG.value]


class Skater(Player):

    ##########################
    # Public Functions
    ##########################

    def gamelog(self, year: str = None) -> list[SkaterGame]:
        if not year:
            year = self._year
        if not self._gamelog_regular_season and not self._gamelog_playoffs:
            self.__populate_gamelogs(year)
        return self._gamelog_regular_season + self._gamelog_playoffs

    def gamelog_playoffs(self, year: str = None) -> list[SkaterGame]:
        if not year:
            year = self._year
        if not self._gamelog_regular_season and not self._gamelog_playoffs:
            self.__populate_gamelogs(year)
        return self._gamelog_playoffs

    def gamelog_regular_season(self, year: str = None) -> list[SkaterGame]:
        if not year:
            year = self._year
        if not self._gamelog_regular_season and not self._gamelog_playoffs:
            self.__populate_gamelogs(year)
        return self._gamelog_regular_season

    def games_played(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.games_played

    def goals(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.goals

    def assists(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.assists

    def points(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.points

    def plus_minus(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.plus_minus

    def penalty_mins(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.penalty_minutes

    def goals_even_strength(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.goals_ev

    def goals_power_play(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.goals_pp

    def goals_short_handed(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.goals_sh

    def goals_game_winning(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.goals_gw

    def shots(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.shots

    def shooting_percentage(self, year: str = None) -> float:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.shooting_pct

    def shifts(self, year: str = None) -> int:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.shifts

    def time_on_ice_per_game(self, year: str = None) -> str:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.time_on_ice_average

    def time_on_ice_total(self, year: str = None) -> str:
        if not self._season_split or year != self._year:
            self._season_split = self.splits(SplitType.SEASON, year)[0]
        return self._season_split.time_on_ice_total

    def splits(self, split_type: SplitType, year: str = None) -> list[SkaterSplit]:
        split_df = self._split_df_lookup(year, split_type)
        return self.__split_df_to_rows(split_df)

    ##########################
    # Gamelog Section
    ##########################

    def __populate_gamelogs(self, year: str) -> None:
        if not self._gamelog_playoffs or not self._gamelog_regular_season:
            soup = self._request_player_gamelog_page(year)
            reg_season_rows = self._regular_season_games(soup)
            playoff_rows = self._playoff_games(soup)
            self._gamelog_regular_season = self.__get_games_by_table_rows(
                reg_season_rows, is_playoff=False
            )
            self._gamelog_playoffs = self.__get_games_by_table_rows(
                playoff_rows, is_playoff=True
            )

    def __get_games_by_table_rows(
        self, rows: ResultSet, is_playoff: bool
    ) -> list[SkaterGame]:
        games = []
        for row_html in rows:
            game = SkaterGame(row_html, is_playoff)
            games += [game]
        return games

    ##########################
    # Splits Section
    ##########################

    def __split_df_to_rows(self, split_df: pd.DataFrame) -> list[SkaterSplit]:
        rows = []
        for index, series in split_df.iterrows():
            rows += [SkaterSplit(series)]
        return rows



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['puck_me',
 'puck_me.game',
 'puck_me.goalie',
 'puck_me.lib',
 'puck_me.player',
 'puck_me.skater']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.9.3,<5.0.0',
 'html5lib>=1.1,<2.0',
 'lxml>=4.6.3,<5.0.0',
 'pandas>=1.3.1,<2.0.0',
 'requests>=2.26.0,<3.0.0']

setup_kwargs = {
    'name': 'puck-me',
    'version': '0.2.0',
    'description': 'Puck Me is a python package for scraping NHL stats from the internet.',
    'long_description': '# Puck Me\n\nPuck Me is a python package for scraping NHL stats from the internet.\n\n## Basic Examples\n\n##### Print skater\'s names:\n\n```\nfrom puck_me.skater.skaters import Skaters\n\nall_skaters = Skaters.all_skaters("2009")\n\n# Print the name of the first three skaters\nfor skater in all_skaters[:3]:\n    print(skater.name)\n```\n\nOutput:\n\n```\n>>> Justin Abdelkader\n>>> Craig Adams\n>>> Maxim Afinogenov\n```\n\n##### Player Basic Stats:\n\n```\nfrom puck_me.skater.skaters import Skaters\n\nall_skaters = Skaters.all_skaters("2009")\n\nfor skater in all_skaters[:3]:\n    name = skater.name\n    games_played = skater.games_played()\n    points = skater.points()\n    print(f"{skater.name} played {games_played} games and scored {points} points.")\n```\n\nOutput:\n\n```\n>>> Justin Abdelkader played 2 games and scored 0 points.\n>>> Craig Adams played 45 games and scored 7 points.\n>>> Maxim Afinogenov played 48 games and scored 20 points.\n```\n\n##### Player Split Stats:\n\n```\nfrom puck_me.skater.skater import SplitType\nfrom puck_me.skater.skaters import Skaters\n\nall_skaters = Skaters.all_skaters("2009")\n\nfor skater in all_skaters[:3]:\n    name = skater.name\n    for arena_split in skater.splits(SplitType.ARENA):\n        value = arena_split.value\n        points = arena_split.points\n        print(f"At arena \'{value}\' {name} scored {points} points.")\n```\n\nOutput:\n\n```\n>>> At arena \'Home\' Justin Abdelkader scored 0 points.\n>>> At arena \'Road\' Justin Abdelkader scored 0 points.\n>>> At arena \'Home\' Craig Adams scored 3 points.\n>>> At arena \'Road\' Craig Adams scored 4 points.\n>>> At arena \'Home\' Maxim Afinogenov scored 12 points.\n>>> At arena \'Road\' Maxim Afinogenov scored 8 points.\n```\n',
    'author': 'Nathan',
    'author_email': 'nathansaccon10@hotmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/nsaccon/hockey_ref_scraper',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
